import { AgGridReact } from 'ag-grid-react';
import { isEqual } from 'lodash';
import * as React from 'react';
import { memo, useMemo, useEffect, useCallback, useState } from 'react';
import styled from 'styled-components';
import { TableDataSource } from '../TableDataSource';
import { translateAgGridFilter, transformColumnDefinitionsIntoAgGrid, } from '../util/ag_translate';
function isEmptyFilter(agFilter) {
    return !agFilter || Object.keys(agFilter).length === 0;
}
export const Grid = memo(function Grid({ slice, columns, filterExpression, agFilterExpression, onFilterChanged, }) {
    const [api, setApi] = useState(undefined);
    const rowData = useMemo(() => new TableDataSource(slice), [slice]);
    const columnDefinitions = useMemo(() => transformColumnDefinitionsIntoAgGrid(columns), [columns]);
    const handleFilterChanged = useCallback((evt) => {
        const filters = evt.api.getFilterModel();
        if (isEmptyFilter(filters)) {
            onFilterChanged(undefined, undefined);
        }
        else {
            const newFilter = translateAgGridFilter(filters, columns);
            if (!isEqual(newFilter, filterExpression)) {
                onFilterChanged(newFilter, filters);
            }
        }
    }, [columns, filterExpression, onFilterChanged]);
    const handleOnGridReady = useCallback(({ api }) => setApi(api), []);
    useEffect(() => {
        if (api) {
            const current = api.getFilterModel();
            // Clear the existing filter
            if (!isEmptyFilter(current) && !filterExpression) {
                api.setFilterModel(null);
                return;
            }
            // Update the existing filter
            if (agFilterExpression && !isEqual(current, agFilterExpression)) {
                api.setFilterModel(agFilterExpression);
                return;
            }
        }
    });
    return (React.createElement(TableContainer, null,
        React.createElement(TableTarget, { className: "ag-theme-dark" },
            React.createElement(AgGridReact, { onGridReady: handleOnGridReady, onFilterChanged: handleFilterChanged, datasource: rowData, rowModelType: "infinite", columnDefs: columnDefinitions }))));
});
const TableContainer = styled.div `
	flex: 1;
`;
const TableTarget = styled.div `
	height: 100%;
	width: 100%;
`;
